/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/libtap.mjs TAP libtap > must match snapshot 1`] = `
Array [
  "Spawn",
  "Stdin",
  "Test",
  "addAssert",
  "afterEach",
  "autoend",
  "bailout",
  "beforeEach",
  "comment",
  "default",
  "doesNotThrow",
  "emits",
  "end",
  "equal",
  "error",
  "expectUncaughtException",
  "fail",
  "fixture",
  "hasStrict",
  "main",
  "match",
  "matchSnapshot",
  "not",
  "notMatch",
  "notOk",
  "notSame",
  "ok",
  "only",
  "pass",
  "plan",
  "pragma",
  "process",
  "processSubtest",
  "rejects",
  "resolveMatch",
  "resolveMatchSnapshot",
  "resolves",
  "same",
  "skip",
  "spawn",
  "stdin",
  "stdinOnly",
  "strictNotSame",
  "strictSame",
  "sub",
  "teardown",
  "test",
  "testdir",
  "throws",
  "throwsArgs",
  "timeout",
  "todo",
  "type",
]
`
