/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/mock.js TAP ok if process missing from the start > must match snapshot 1`] = `
TAP version 13
1..1
# Subtest: child
    1..1
    ok 1 - one is one
ok 1 - child


`
