/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/waiter.js TAP abort after reject > next tick 1`] = `
Object {
  "done": true,
  "expectReject": true,
  "finishing": true,
  "ready": true,
  "rejected": true,
  "resolved": false,
  "value": "reject",
}
`

exports[`test/waiter.js TAP abort after reject > same tick 1`] = `
Object {
  "done": false,
  "expectReject": true,
  "finishing": false,
  "ready": true,
  "rejected": false,
  "resolved": false,
  "value": null,
}
`

exports[`test/waiter.js TAP abort after resolve > next tick 1`] = `
Object {
  "done": true,
  "expectReject": false,
  "finishing": true,
  "ready": true,
  "rejected": false,
  "resolved": true,
  "value": "resolve",
}
`

exports[`test/waiter.js TAP abort after resolve > same tick 1`] = `
Object {
  "done": false,
  "expectReject": false,
  "finishing": false,
  "ready": true,
  "rejected": false,
  "resolved": false,
  "value": null,
}
`

exports[`test/waiter.js TAP abort reject > next tick 1`] = `
Object {
  "done": true,
  "expectReject": true,
  "finishing": true,
  "ready": true,
  "rejected": false,
  "resolved": false,
  "value": "abort",
}
`

exports[`test/waiter.js TAP abort reject > same tick 1`] = `
Object {
  "done": true,
  "expectReject": true,
  "finishing": true,
  "ready": true,
  "rejected": false,
  "resolved": false,
  "value": "abort",
}
`

exports[`test/waiter.js TAP abort resolve > next tick 1`] = `
Object {
  "done": true,
  "expectReject": false,
  "finishing": true,
  "ready": true,
  "rejected": true,
  "resolved": false,
  "value": "abort",
}
`

exports[`test/waiter.js TAP abort resolve > same tick 1`] = `
Object {
  "done": true,
  "expectReject": false,
  "finishing": true,
  "ready": true,
  "rejected": true,
  "resolved": false,
  "value": "abort",
}
`

exports[`test/waiter.js TAP expected reject > next tick 1`] = `
Object {
  "done": true,
  "expectReject": true,
  "finishing": true,
  "ready": true,
  "rejected": true,
  "resolved": false,
  "value": "reject",
}
`

exports[`test/waiter.js TAP expected reject > same tick 1`] = `
Object {
  "done": false,
  "expectReject": true,
  "finishing": false,
  "ready": true,
  "rejected": false,
  "resolved": false,
  "value": null,
}
`

exports[`test/waiter.js TAP expected resolve > next tick 1`] = `
Object {
  "done": true,
  "expectReject": false,
  "finishing": true,
  "ready": true,
  "rejected": false,
  "resolved": true,
  "value": "resolve",
}
`

exports[`test/waiter.js TAP expected resolve > same tick 1`] = `
Object {
  "done": false,
  "expectReject": false,
  "finishing": false,
  "ready": true,
  "rejected": false,
  "resolved": false,
  "value": null,
}
`

exports[`test/waiter.js TAP finish after reject > next tick 1`] = `
Object {
  "done": true,
  "expectReject": true,
  "finishing": true,
  "ready": true,
  "rejected": true,
  "resolved": false,
  "value": "reject",
}
`

exports[`test/waiter.js TAP finish after reject > same tick 1`] = `
Object {
  "done": false,
  "expectReject": true,
  "finishing": false,
  "ready": true,
  "rejected": false,
  "resolved": false,
  "value": null,
}
`

exports[`test/waiter.js TAP finish after resolve > next tick 1`] = `
Object {
  "done": true,
  "expectReject": false,
  "finishing": true,
  "ready": true,
  "rejected": false,
  "resolved": true,
  "value": "resolve",
}
`

exports[`test/waiter.js TAP finish after resolve > same tick 1`] = `
Object {
  "done": false,
  "expectReject": false,
  "finishing": false,
  "ready": true,
  "rejected": false,
  "resolved": false,
  "value": null,
}
`

exports[`test/waiter.js TAP unexpected reject > next tick 1`] = `
Object {
  "done": true,
  "expectReject": false,
  "finishing": true,
  "ready": true,
  "rejected": true,
  "resolved": false,
  "value": "reject",
}
`

exports[`test/waiter.js TAP unexpected reject > same tick 1`] = `
Object {
  "done": false,
  "expectReject": false,
  "finishing": false,
  "ready": true,
  "rejected": false,
  "resolved": false,
  "value": null,
}
`

exports[`test/waiter.js TAP unexpected resolve > next tick 1`] = `
Object {
  "done": true,
  "expectReject": true,
  "finishing": true,
  "ready": true,
  "rejected": false,
  "resolved": true,
  "value": "resolve",
}
`

exports[`test/waiter.js TAP unexpected resolve > same tick 1`] = `
Object {
  "done": false,
  "expectReject": true,
  "finishing": false,
  "ready": true,
  "rejected": false,
  "resolved": false,
  "value": null,
}
`
