/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/settings.js TAP > must match snapshot 1`] = `
Object {
  "atTap": false,
  "mkdirpNeeded": "version specific",
  "mkdirRecursive": Function mkdirRecursive(dir, cb),
  "mkdirRecursiveSync": Function mkdirRecursiveSync(dir),
  "output": "process.stdout",
  "rimrafNeeded": "version specific",
  "rmdirRecursive": Function rmdirRecursive(dir, cb),
  "rmdirRecursiveSync": Function rmdirRecursiveSync(dir),
  "snapshotFile": Function snapshotFile(cwd, main, argv),
  "stackUtils": Object {
    "ignoredPackages": Array [],
    "internals": Array [],
  },
  "StackUtils": Function StackUtils(classStackUtils),
}
`
