/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/tap/resume.js TAP > exit status 1`] = `
Object {
  "code": 0,
  "signal": null,
}
`

exports[`test/tap/resume.js TAP > stderr 1`] = `

`

exports[`test/tap/resume.js TAP > stdout 1`] = `
1
ok 1 - 2
3
ok 2 - 4
ok 3 - 5
1..3
# {time}

`
